
class MemoryStats(object):
    _proc_status = '/proc/self/status'

    _scale = {'kB': 1024.0, 'mB': 1024.0*1024.0,
              'KB': 1024.0, 'MB': 1024.0*1024.0}

    def _VmB(self, VmKey):
        '''Private.
        '''
        # get pseudo file  /proc/<pid>/status
        try:
            t = open(self._proc_status)
            v = t.read()
            t.close()
        except:
            return 0.0  # non-Linux?
            # get VmKey line e.g. 'VmRSS:  9999  kB\n ...'
        i = v.index(VmKey)
        v = v[i:].split(None, 3)  # whitespace
        if len(v) < 3:
            return 0.0  # invalid format?
            # convert Vm value to bytes
        return float(v[1]) * self._scale[v[2]]


    def memory(self, since=0.0):
        '''Return memory usage in bytes.
        '''
        return self._VmB('VmSize:') - since

    def resident(self, since=0.0):
        '''Return resident memory usage in bytes.
        '''
        return self._VmB('VmRSS:') - since

    def stacksize(self, since=0.0):
        '''Return stack size in bytes.
        '''
        return self._VmB('VmStk:') - since



import logging
import tornado
from django.core.management.base import BaseCommand
from django_sockjs_server.lib.config import SockJSSereverSettings
from django_sockjs_server.lib.sockjs_handler import SockJSRouterPika, SockJSConnection, StatsHandler


class Command(BaseCommand):
    def handle(self, *args, **options):
        logger = logging.getLogger(__name__)
        logger.info('start django-sockjs-server')
        self.config = SockJSSereverSettings()

        io_loop = tornado.ioloop.IOLoop.instance()

        router = SockJSRouterPika(
            SockJSConnection,
            self.config.listen_location,
            user_settings=self.config.router_settings)
        app = tornado.web.Application([(r"/stats/(.*)", StatsHandler, dict(pika_client=router._connection.pika_client))] +
                                      router.urls)



        app.listen(self.config.listen_port, address=self.config.listen_addr)
        try:
            io_loop.start()
        except KeyboardInterrupt:
            pass



from django.conf import settings

__author__ = 'Sergey Kravchuk'

class SockJSSereverSettings(object):

    def __init__(self):
        conf = getattr(settings, 'DJANGO_SOCKJS_SERVER', None)
        if not conf:
            raise Exception('django-sockjs-server error! No settings.')

        self.rabbitmq_user = conf.get('rabbitmq_user', 'guest')
        self.rabbitmq_password = conf.get('rabbitmq_password', 'guest')
        self.rabbitmq_host = conf.get('rabbitmq_server_host', 'localhost')
        self.rabbitmq_port = int(conf.get('rabbitmq_server_port', 5672))
        self.rabbitmq_vhost = conf.get('rabbitmq_server_vhost', '/')
        self.rabbitmq_exhange_name = conf.get('rabbitmq_exhange_name', 'sockjs')
        self.rabbitmq_exchange_type = conf.get('rabbitmq_exchange_type', 'fanout')

        self.listen_addr = conf.get('listen_addr', '0.0.0.0')
        self.listen_port = int(conf.get('listen_port', 8083))
        self.listen_location = conf.get('listen_location', '/ws')
        self.secret_key = conf.get('secret_key', 'not_set_secret_key')
        self.sockjs_url = conf.get('sockjs_url', ['http://localhost:8083/ws'])

        self.router_settings = conf.get('router_settings', dict())



from collections import defaultdict
import json
import logging
from django.conf import settings
from django.utils.timezone import now
import pika
from pika.adapters.tornado_connection import TornadoConnection
from pika.exceptions import AMQPConnectionError
import time
from django_sockjs_server.lib.config import SockJSSereverSettings


class PikaClient(object):
    def __init__(self, io_loop):
        self.logger = logging.getLogger(__name__)
        self.logger.info('PikaClient: __init__')
        self.io_loop = io_loop

        self.connected = False
        self.connecting = False
        self.connection = None
        self.channel = None

        self.event_listeners_count = 0
        self.event_listeners = set()
        self.subscrib_channel = defaultdict(set)
        self.last_reconnect = now()
        self.uptime_start = now()



        self.config = SockJSSereverSettings()

    def connect(self):
        if self.connecting:
            self.logger.info('django-sockjs-server(PikaClient): Already connecting to RabbitMQ')
            return

        self.logger.info('django-sockjs-server(PikaClient): Connecting to RabbitMQ')
        self.connecting = True

        cred = pika.PlainCredentials(self.config.rabbitmq_user, self.config.rabbitmq_password)
        param = pika.ConnectionParameters(
            host=self.config.rabbitmq_host,
            port=self.config.rabbitmq_port,
            virtual_host=self.config.rabbitmq_vhost,
            credentials=cred
        )

        try:
            self.connection = TornadoConnection(param,
                                                on_open_callback=self.on_connected)
            self.connection.add_on_close_callback(self.on_closed)
        except AMQPConnectionError:
            self.logger.info('django-sockjs-server(PikaClient): error connect, wait 5 sec')
            time.sleep(5)
            self.reconnect()

        self.last_reconnect = now()

    def on_connected(self, connection):
        self.logger.info('django-sockjs-server(PikaClient): connected to RabbitMQ')
        self.connected = True
        self.connection = connection
        self.connection.channel(self.on_channel_open)

    def on_channel_open(self, channel):
        self.logger.info('django-sockjs-server(PikaClient): Channel open, Declaring exchange')
        self.channel = channel
        self.channel.exchange_declare(exchange=self.config.rabbitmq_exhange_name,
                                      exchange_type=self.config.rabbitmq_exchange_type)
        self.channel.queue_declare(exclusive=False, auto_delete=True, callback=self.on_queue_declared)

    def on_queue_declared(self, frame):
        self.logger.info('django-sockjs-server(PikaClient): queue bind')
        self.channel.queue_bind(callback=None, exchange=self.config.rabbitmq_exhange_name, queue=frame.method.queue)
        self.channel.basic_consume(self.handle_delivery, queue=frame.method.queue, no_ack=True)

    def handle_delivery(self, channel, method, header, body):
        """Called when we receive a message from RabbitMQ"""
        self.notify_listeners(body)

    def on_closed(self, connection, error_code, error_message):
        self.logger.info('django-sockjs-server(PikaClient): rabbit connection closed, wait 5 seconds')
        connection.add_timeout(5, self.reconnect)

    def reconnect(self):
        self.connecting = False
        self.logger.info('django-sockjs-server(PikaClient): reconnect')
        self.connect()

    def notify_listeners(self, event_json):
        event_obj = json.loads(event_json)

        self.logger.debug('django-sockjs-server(PikaClient): get new data  = %s ' % event_obj)
        try:
            if len(self.subscrib_channel[event_obj['channel']]) > 0:
                for client in self.subscrib_channel[event_obj['channel']]:
                    self.logger.debug('django-sockjs-server(PikaClient): send message channel = %s ' % event_obj['channel'])
                    client.broadcast(self.subscrib_channel[event_obj['channel']], event_json)
                    break
        except KeyError:
            pass

    def add_event_listener(self, listener):
        self.event_listeners.add(listener)
        self.event_listeners_count += 1
        self.logger.debug('django-sockjs-server(PikaClient): listener %s added' % repr(listener))

    def remove_event_listener(self, listener):
        try:
            self.event_listeners.remove(listener)
            self.event_listeners_count -= 1
            self.logger.debug('django-sockjs-server(PikaClient): listener %s removed' % repr(listener))
        except KeyError:
            pass

    def add_subscriber_channel(self, chanel, client):
        self.subscrib_channel[chanel].add(client)
        self.logger.debug('django-sockjs-server(PikaClient): listener %s add to channel %s' % (repr(client), chanel))

    def remove_subscriber_channel(self, chanel, client):
        try:
            self.subscrib_channel[chanel].remove(client)
            self.logger.debug('django-sockjs-server(PikaClient): listener %s remove from channel %s' % (repr(client),
                              chanel))
        except KeyError:
            pass

    def get_event_listeners_count(self):
        return self.event_listeners_count

    def get_subscribe_channel_count(self):
        return len(self.subscrib_channel.keys())

    def get_subscribe_channels(self):
        return self.subscrib_channel.keys()

    def get_last_reconnect(self):
        return self.last_reconnect

    def get_uptime(self):
        return (now() - self.uptime_start).seconds


from random import choice
from django import template
from django_sockjs_server.lib.config import SockJSSereverSettings
from django_sockjs_server.lib.token import Token

register = template.Library()

@register.simple_tag(name='sockjs_auth_token')
def sockjs_auth_token(channel_name, unq_id=None):
    token = Token()
    if unq_id:
        return token.get_secret_data(channel_name+str(unq_id))
    return token.get_secret_data(channel_name)


@register.simple_tag(name='sockjs_server_url')
def sockjs_server_url():
    config = SockJSSereverSettings()
    return choice(config.sockjs_url)


import json
import logging
from django_sockjs_server.lib.token import Token

class Subscribe(object):

    def __init__(self, connection):
        self.conn = connection
        self.logger = logging.getLogger(__name__)

    def add(self, data):
        try:
            json_obj = json.loads(data)
            token = Token()
            if token.get_data(json_obj['token'], json_obj['data']['channel']):
                self.logger.debug('django-sockjs-server:Subscrbe to channel %s' % json_obj['data']['channel'])
                self.conn.subscribe_list.add(json_obj['data']['channel'])
                self.conn.pika_client.add_subscriber_channel(json_obj['data']['channel'], self.conn)
        except (KeyError, TypeError):
            pass

    def remove(self):
        for channel in self.conn.subscribe_list:
            self.conn.pika_client.remove_subscriber_channel(channel, self.conn)
        self.conn.subscribe_list = None

    def is_access(self, token):

        pass




import logging
from django.utils.timezone import now
import sockjs.tornado
from tornado.web import RequestHandler
from django_sockjs_server.lib.memory_stats import MemoryStats
from django_sockjs_server.lib.pika_client import PikaClient
from django_sockjs_server.lib.subscribe import Subscribe

class SockJSConnection(sockjs.tornado.SockJSConnection):

    pika_client = None  # should be initialized by router

    def __init__(self, *args, **kw):
        self.subscribe_list = set()
        self.subscribe = Subscribe(self)
        self.logger = logging.getLogger(__name__)
        super(SockJSConnection, self).__init__(*args, **kw)

    def on_open(self, info):
        self.pika_client.add_event_listener(self)

    def on_close(self):
        self.subscribe.remove()
        self.pika_client.remove_event_listener(self)

    def on_message(self, message):
        self.logger.debug('Get message %s' % message)
        self.subscribe.add(message)

class StatsHandler(RequestHandler):
    def initialize(self, pika_client):
        self.pika_client = pika_client
        self.memory_stats = MemoryStats()

    def get(self, type_stats='default'):
        self.clear()
        self.set_header("Content-Type", "text/plain")
        self.set_status(200)
        if type_stats == 'debug':
            self.finish("uptime_seconds: " + str(self.pika_client.get_uptime()) +
                        "\n memory_use_byte: " + str(int(self.memory_stats.memory())) +
                        "\n memory_resident_use_byte: " + str(int(self.memory_stats.resident())) +
                        "\n memory_stack_size_byte: " + str(int(self.memory_stats.stacksize())) +
                        "\n last_rabbitmq_reconnect: " + str(self.pika_client.get_last_reconnect()) +
                        "\n connect_rabbitmq_time_seconds: " + str((now() - self.pika_client.get_last_reconnect()).seconds) +
                        "\n connects: " + str(self.pika_client.get_event_listeners_count()) +
                        "\n channel_count: " + str(len(self.pika_client.get_subscribe_channels())) +
                        "\n channels: " + str(self.pika_client.get_subscribe_channels()))
        else:
            self.finish("uptime_seconds: " + str(self.pika_client.get_uptime()) +
                        "\n memory_use_byte: " + str(int(self.memory_stats.memory())) +
                        "\n memory_resident_use_byte: " + str(int(self.memory_stats.resident())) +
                        "\n memory_stack_size_byte: " + str(int(self.memory_stats.stacksize())) +
                        "\n last_rabbitmq_reconnect: " + str(self.pika_client.get_last_reconnect()) +
                        "\n connect_rabbitmq_time_seconds: " + str((now() - self.pika_client.get_last_reconnect()).seconds) +
                        "\n connects: " + str(self.pika_client.get_event_listeners_count()) +
                        "\n channel_count: " + str(len(self.pika_client.get_subscribe_channels())))


class SockJSRouterPika(sockjs.tornado.SockJSRouter):
    def __init__(self, *args, **kw):
        super(SockJSRouterPika, self).__init__(*args, **kw)
        self._connection.pika_client = PikaClient(self.io_loop)
        self._connection.pika_client.connect()


import logging
import time
import json
import pika
from pika.exceptions import ChannelClosed, AMQPConnectionError
from django.core.serializers.json import DjangoJSONEncoder
from django_sockjs_server.lib.config import SockJSSereverSettings


class SockJsServerClient(object):
    def __init__(self):
        self.logger = logging.getLogger(__name__)
        self.config = SockJSSereverSettings()
        self.connected = False
        self.retry_count = 0

    def _connect(self, is_retry=False):
        cred = pika.PlainCredentials(self.config.rabbitmq_user, self.config.rabbitmq_password)
        param = pika.ConnectionParameters(
            host=self.config.rabbitmq_host,
            port=self.config.rabbitmq_port,
            virtual_host=self.config.rabbitmq_vhost,
            credentials=cred
        )
        self.connection = pika.BlockingConnection(param)
        self.channel = self.connection.channel()
        self.channel.exchange_declare(exchange=self.config.rabbitmq_exhange_name,
                                      exchange_type=self.config.rabbitmq_exchange_type)
        self.connected = True
        if not is_retry:
            self.retry_count = 0

    def _disconnect(self):
        self.connected = False
        self.connection.disconnect()

    def publish_message(self, message, is_retry=False):
        try:
            if not self.connected:
                self._connect(is_retry)
            self.channel.basic_publish(self.config.rabbitmq_exhange_name,
                                       routing_key='',
                                       body=json.dumps(message, cls=DjangoJSONEncoder))
        except (ChannelClosed, AMQPConnectionError):
            if self.connected:
                self._disconnect()
            if self.retry_count < 4:
                self.retry_count += 1
                #wait 100 ms
                time.sleep(100 / 1000000.0)
                self.publish_message(message, True)



from hashlib import md5
from django_sockjs_server.lib.config import SockJSSereverSettings

class Token(object):
    def __init__(self):
        self.config = SockJSSereverSettings()

    def get_secret_data(self, data):
        token = self.config.secret_key + data
        return "%s" % md5(token).hexdigest()

    def get_data(self, token, data):
        if token == self.get_secret_data(data):
            return data
        raise KeyError



from distutils.core import setup
import os

version = "0.1.9"

# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir:
    os.chdir(root_dir)

for dirpath, dirnames, filenames in os.walk('django_sockjs_server'):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'): del dirnames[i]
    if '__init__.py' in filenames:
        pkg = dirpath.replace(os.path.sep, '.')
        if os.path.altsep:
            pkg = pkg.replace(os.path.altsep, '.')
        packages.append(pkg)
    elif filenames:
        prefix = dirpath[13:]
        for f in filenames:
            data_files.append(os.path.join(prefix, f))


setup(name='django-sockjs-server',
      version=version,
      description='SockJS server for Django',
      author='Sergey Kravchuk',
      author_email='alfss.obsd@gmail.com',
      url='https://github.com/alfss/django-sockjs-server',
      download_url='https://github.com/alfss/django-sockjs-server/archive/0.1.9.tar.gz',
      package_dir={'django_sockjs_server': 'django_sockjs_server'},
      packages=packages,
      install_requires=['sockjs-tornado >= 1.0.0',
                        'pika >= 0.9.12'],
      package_data={'django_sockjs_server': data_files},
      classifiers=['Development Status :: 5 - Production/Stable',
                   'Environment :: Web Environment',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Internet :: WWW/HTTP :: HTTP Servers'],
      zip_safe=False,
)
